@@ -26,19 +26,22 @@
 #include <glib.h>
 #include "file-data.h"
 #include "file-utils.h"
-#include "glib-utils.h"
-#include "fr-process.h"
 #include "fr-command.h"
+#include "fr-enum-types.h"
 #include "fr-marshal.h"
+#include "fr-process.h"
+#include "glib-utils.h"
 
 #define INITIAL_SIZE 256
 
+
 /* Signals */
 enum {
 	START,
 	DONE,
 	PROGRESS,
 	MESSAGE,
+	WORKING_ARCHIVE,
 	LAST_SIGNAL
 };
 
@@ -47,7 +50,11 @@ enum {
         PROP_0,
         PROP_FILENAME,
         PROP_MIME_TYPE,
-        PROP_PROCESS
+        PROP_PROCESS,
+        PROP_PASSWORD,
+        PROP_ENCRYPT_HEADER,
+        PROP_COMPRESSION,
+        PROP_VOLUME_SIZE
 };
 
 static GObjectClass *parent_class = NULL;
@@ -58,17 +65,17 @@ static void fr_command_init        (FrCommand *afile);
 static void fr_command_finalize    (GObject *object);
 
 char *action_names[] = { "NONE",
-			 "CREATING_NEW_ARCHIVE", 
-			 "LOADING_ARCHIVE", 
-			 "LISTING_CONTENT", 
-			 "DELETING_FILES", 
-			 "TESTING_ARCHIVE", 
-			 "GETTING_FILE_LIST", 
-			 "COPYING_FILES_FROM_REMOTE", 
-			 "ADDING_FILES", 
-			 "EXTRACTING_FILES", 
-			 "COPYING_FILES_TO_REMOTE", 
-			 "CREATING_ARCHIVE", 
+			 "CREATING_NEW_ARCHIVE",
+			 "LOADING_ARCHIVE",
+			 "LISTING_CONTENT",
+			 "DELETING_FILES",
+			 "TESTING_ARCHIVE",
+			 "GETTING_FILE_LIST",
+			 "COPYING_FILES_FROM_REMOTE",
+			 "ADDING_FILES",
+			 "EXTRACTING_FILES",
+			 "COPYING_FILES_TO_REMOTE",
+			 "CREATING_ARCHIVE",
 			 "SAVING_REMOTE_ARCHIVE" };
 
 GType
@@ -100,45 +107,44 @@ fr_command_get_type ()
 
 
 static void
-base_fr_command_list (FrCommand  *comm,
-		      const char *password)
+base_fr_command_list (FrCommand  *comm)
 {
 }
 
 
 static void
 base_fr_command_add (FrCommand     *comm,
+		     const char    *from_file,
 		     GList         *file_list,
 		     const char    *base_dir,
 		     gboolean       update,
-		     const char    *password,
-		     FrCompression  compression)
+		     gboolean       recursive)
 {
 }
 
 
 static void
-base_fr_command_delete (FrCommand *comm,
-			GList     *file_list)
+base_fr_command_delete (FrCommand  *comm,
+		        const char *from_file,
+			GList       *file_list)
 {
 }
 
 
 static void
 base_fr_command_extract (FrCommand  *comm,
+		         const char *from_file,
 			 GList      *file_list,
 			 const char *dest_dir,
 			 gboolean    overwrite,
 			 gboolean    skip_older,
-			 gboolean    junk_paths,
-			 const char *password)
+			 gboolean    junk_paths)
 {
 }
 
 
 static void
-base_fr_command_test (FrCommand   *comm,
-		      const char  *password)
+base_fr_command_test (FrCommand *comm)
 {
 }
 
@@ -150,32 +156,42 @@ base_fr_command_uncompress (FrCommand *comm)
 
 
 static void
-base_fr_command_recompress (FrCommand     *comm,
-			    FrCompression  compression)
+base_fr_command_recompress (FrCommand *comm)
 {
 }
 
 
-char *
-base_fr_command_escape (FrCommand     *comm,
-			const char    *str)
-{
-	return shell_escape (str);
-}
-
-
 static void
-base_fr_command_handle_error (FrCommand *comm,
+base_fr_command_handle_error (FrCommand   *comm,
 			      FrProcError *error)
 {
 }
 
 
+const char **void_mime_types = { NULL };
+
+
+const char **
+base_fr_command_get_mime_types (FrCommand *comm)
+{
+	return void_mime_types;
+}
+
+
+FrCommandCap
+base_fr_command_get_capabilities (FrCommand  *comm,
+			          const char *mime_type)
+{
+	return FR_COMMAND_CAN_DO_NOTHING;
+}
+
+
 static void
 base_fr_command_set_mime_type (FrCommand  *comm,
 			       const char *mime_type)
 {
 	comm->mime_type = get_static_string (mime_type);
+	fr_command_update_capabilities (comm);
 }
 
 
@@ -184,9 +200,9 @@ fr_command_start (FrProcess *process,
 		  gpointer   data)
 {
 	FrCommand *comm = FR_COMMAND (data);
-	
+
 	g_signal_emit (G_OBJECT (comm),
-		       fr_command_signals[START], 
+		       fr_command_signals[START],
 		       0,
 		       comm->action);
 }
@@ -206,31 +222,31 @@ fr_command_done (FrProcess   *process,
 		fr_process_start (comm->process);
 	else
 		g_signal_emit (G_OBJECT (comm),
-			       fr_command_signals[DONE], 
+			       fr_command_signals[DONE],
 			       0,
-			       comm->action, 
+			       comm->action,
 			       &process->error);
 }
 
 
 static void
-fr_command_set_process (FrCommand  *comm,
-		        FrProcess  *process)
+fr_command_set_process (FrCommand *comm,
+		        FrProcess *process)
 {
 	if (comm->process != NULL) {
 		g_signal_handlers_disconnect_matched (G_OBJECT (comm->process),
-					      G_SIGNAL_MATCH_DATA, 
-					      0, 
-					      0, NULL, 
+					      G_SIGNAL_MATCH_DATA,
+					      0,
+					      0, NULL,
 					      0,
 					      comm);
 		g_object_unref (G_OBJECT (comm->process));
 		comm->process = NULL;
 	}
-	
+
 	if (process == NULL)
 		return;
-	
+
 	g_object_ref (G_OBJECT (process));
 	comm->process = process;
 	g_signal_connect (G_OBJECT (comm->process),
@@ -250,23 +266,36 @@ fr_command_set_property (GObject      *object,
 			 const GValue *value,
 			 GParamSpec   *pspec)
 {
-        FrCommand *comm;
+	FrCommand *comm;
 
-        comm = FR_COMMAND (object);
+	comm = FR_COMMAND (object);
 
-        switch (prop_id) {
-        case PROP_PROCESS:
-                fr_command_set_process (comm, g_value_get_object (value));
-                break;
-        case PROP_FILENAME:
-        	fr_command_set_filename (comm, g_value_get_string (value));
-                break;
-        case PROP_MIME_TYPE:
-        	fr_command_set_mime_type (comm, g_value_get_string (value));
-                break;
-        default:
-                break;
-        }
+	switch (prop_id) {
+	case PROP_PROCESS:
+		fr_command_set_process (comm, g_value_get_object (value));
+		break;
+	case PROP_FILENAME:
+		fr_command_set_filename (comm, g_value_get_string (value));
+		break;
+	case PROP_MIME_TYPE:
+		fr_command_set_mime_type (comm, g_value_get_string (value));
+		break;
+	case PROP_PASSWORD:
+		g_free (comm->password);
+		comm->password = g_strdup (g_value_get_string (value));
+		break;
+	case PROP_ENCRYPT_HEADER:
+		comm->encrypt_header = g_value_get_boolean (value);
+		break;
+	case PROP_COMPRESSION:
+		comm->compression = g_value_get_enum (value);
+		break;
+	case PROP_VOLUME_SIZE:
+		comm->volume_size = g_value_get_uint (value);
+		break;
+	default:
+		break;
+	}
 }
 
 
@@ -276,24 +305,36 @@ fr_command_get_property (GObject    *object,
 			 GValue     *value,
 			 GParamSpec *pspec)
 {
-        FrCommand *comm;
+	FrCommand *comm;
 
-        comm = FR_COMMAND (object);
+	comm = FR_COMMAND (object);
 
-        switch (prop_id) {
-        case PROP_PROCESS:
-                g_value_set_object (value, comm->process);
-                break;
-        case PROP_FILENAME:
-        	g_value_set_string (value, comm->filename);
-                break;
-        case PROP_MIME_TYPE:
-                g_value_set_static_string (value, comm->mime_type);
-                break;
-        default:
-                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-                break;
-        }
+	switch (prop_id) {
+	case PROP_PROCESS:
+		g_value_set_object (value, comm->process);
+		break;
+	case PROP_FILENAME:
+		g_value_set_string (value, comm->filename);
+		break;
+	case PROP_MIME_TYPE:
+		g_value_set_static_string (value, comm->mime_type);
+		break;
+	case PROP_PASSWORD:
+		g_value_set_string (value, comm->password);
+		break;
+	case PROP_ENCRYPT_HEADER:
+		g_value_set_boolean (value, comm->encrypt_header);
+		break;
+	case PROP_COMPRESSION:
+		g_value_set_enum (value, comm->compression);
+		break;
+	case PROP_VOLUME_SIZE:
+		g_value_set_uint (value, comm->volume_size);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
 }
 
 
@@ -305,30 +346,31 @@ fr_command_class_init (FrCommandClass *class)
 	parent_class = g_type_class_peek_parent (class);
 
 	gobject_class = G_OBJECT_CLASS (class);
-	
+
 	/* virtual functions */
-	
+
 	gobject_class->finalize = fr_command_finalize;
 	gobject_class->set_property = fr_command_set_property;
-        gobject_class->get_property = fr_command_get_property;
-               
-	class->list           = base_fr_command_list;
-	class->add            = base_fr_command_add;
-	class->delete         = base_fr_command_delete;
-	class->extract        = base_fr_command_extract;
-	class->test           = base_fr_command_test;
-	class->uncompress     = base_fr_command_uncompress;
-	class->recompress     = base_fr_command_recompress;
-	class->escape         = base_fr_command_escape;
-	class->handle_error   = base_fr_command_handle_error;
-	class->set_mime_type  = base_fr_command_set_mime_type;
-	class->start          = NULL;
-	class->done           = NULL;
-	class->progress       = NULL;
-	class->message        = NULL;
-	
+	gobject_class->get_property = fr_command_get_property;
+
+	class->list             = base_fr_command_list;
+	class->add              = base_fr_command_add;
+	class->delete           = base_fr_command_delete;
+	class->extract          = base_fr_command_extract;
+	class->test             = base_fr_command_test;
+	class->uncompress       = base_fr_command_uncompress;
+	class->recompress       = base_fr_command_recompress;
+	class->handle_error     = base_fr_command_handle_error;
+	class->get_mime_types   = base_fr_command_get_mime_types;
+	class->get_capabilities = base_fr_command_get_capabilities;
+	class->set_mime_type    = base_fr_command_set_mime_type;
+	class->start            = NULL;
+	class->done             = NULL;
+	class->progress         = NULL;
+	class->message          = NULL;
+
 	/* signals */
-	
+
 	fr_command_signals[START] =
 		g_signal_new ("start",
 			      G_TYPE_FROM_CLASS (class),
@@ -336,7 +378,7 @@ fr_command_class_init (FrCommandClass *class)
 			      G_STRUCT_OFFSET (FrCommandClass, start),
 			      NULL, NULL,
 			      fr_marshal_VOID__INT,
-			      G_TYPE_NONE, 
+			      G_TYPE_NONE,
 			      1, G_TYPE_INT);
 	fr_command_signals[DONE] =
 		g_signal_new ("done",
@@ -366,13 +408,22 @@ fr_command_class_init (FrCommandClass *class)
 			      fr_marshal_VOID__STRING,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_STRING);
-			      
+	fr_command_signals[WORKING_ARCHIVE] =
+			g_signal_new ("working_archive",
+				      G_TYPE_FROM_CLASS (class),
+				      G_SIGNAL_RUN_LAST,
+				      G_STRUCT_OFFSET (FrCommandClass, working_archive),
+				      NULL, NULL,
+				      fr_marshal_VOID__STRING,
+				      G_TYPE_NONE, 1,
+				      G_TYPE_STRING);
+
 	/* properties */
-	
+
 	g_object_class_install_property (gobject_class,
 					 PROP_PROCESS,
 					 g_param_spec_object ("process",
-							      "Processs",
+							      "Process",
 							      "The process object used by the command",
 							      FR_TYPE_PROCESS,
 							      G_PARAM_READWRITE));
@@ -389,7 +440,38 @@ fr_command_class_init (FrCommandClass *class)
 							      "Mime type",
 							      "The file mime-type",
 							      NULL,
-							      G_PARAM_READWRITE));	
+							      G_PARAM_READWRITE));
+	g_object_class_install_property (gobject_class,
+					 PROP_PASSWORD,
+					 g_param_spec_string ("password",
+							      "Password",
+							      "The archive password",
+							      NULL,
+							      G_PARAM_READWRITE));
+	g_object_class_install_property (gobject_class,
+					 PROP_ENCRYPT_HEADER,
+					 g_param_spec_boolean ("encrypt-header",
+							       "Encrypt header",
+							       "Whether to encrypt the archive header when creating the archive",
+							       FALSE,
+							       G_PARAM_READWRITE));
+	g_object_class_install_property (gobject_class,
+					 PROP_COMPRESSION,
+					 g_param_spec_enum ("compression",
+							    "Compression type",
+							    "The compression type to use when creating the archive",
+							    FR_TYPE_COMPRESSION,
+							    FR_COMPRESSION_NORMAL,
+							    G_PARAM_READWRITE));
+	g_object_class_install_property (gobject_class,
+					 PROP_VOLUME_SIZE,
+					 g_param_spec_uint ("volume-size",
+							    "Volume size",
+							    "The size of each volume or 0 to not use volumes",
+							    0L,
+							    G_MAXUINT,
+							    0,
+							    G_PARAM_READWRITE));
 }
 
 
@@ -397,12 +479,15 @@ static void
 fr_command_init (FrCommand *comm)
 {
 	comm->files = g_ptr_array_sized_new (INITIAL_SIZE);
-	
+
+	comm->password = NULL;
+	comm->encrypt_header = FALSE;
+	comm->compression = FR_COMPRESSION_NORMAL;
+	comm->volume_size = 0;
 	comm->filename = NULL;
 	comm->e_filename = NULL;
 	comm->fake_load = FALSE;
 
-	comm->propCanModify = TRUE;
 	comm->propAddCanUpdate = FALSE;
 	comm->propAddCanReplace = FALSE;
 	comm->propAddCanStoreFolders = FALSE;
@@ -411,9 +496,10 @@ fr_command_init (FrCommand *comm)
 	comm->propExtractCanJunkPaths = FALSE;
 	comm->propPassword = FALSE;
 	comm->propTest = FALSE;
-	comm->propCanExtractAll = TRUE; 	
+	comm->propCanExtractAll = TRUE;
 	comm->propCanDeleteNonEmptyFolders = TRUE;
 	comm->propCanExtractNonEmptyFolders = TRUE;
+	comm->propListFromFile = FALSE;
 }
 
 
@@ -427,11 +513,10 @@ fr_command_finalize (GObject *object)
 
 	comm = FR_COMMAND (object);
 
-	if (comm->filename != NULL)
-		g_free (comm->filename);
-	if (comm->e_filename != NULL)
-		g_free (comm->e_filename);
-	if (comm->files != NULL) 
+	g_free (comm->filename);
+	g_free (comm->e_filename);
+	g_free (comm->password);
+	if (comm->files != NULL)
 		g_ptr_array_free_full (comm->files, (GFunc) file_data_free, NULL);
 	fr_command_set_process (comm, NULL);
 
@@ -456,33 +541,42 @@ fr_command_set_filename (FrCommand  *comm,
 		g_free (comm->e_filename);
 		comm->e_filename = NULL;
 	}
-	
-	if (filename == NULL)
-		return;
 
-	if (! g_path_is_absolute (filename)) {
-		char *current_dir;
+	if (filename != NULL) {
+		if (! g_path_is_absolute (filename)) {
+			char *current_dir;
 
-		current_dir = g_get_current_dir ();
-		comm->filename = g_strconcat (current_dir,
-					      "/", 
-					      filename, 
-					      NULL);
-		g_free (current_dir);
-	} 
-	else
-		comm->filename = g_strdup (filename);
+			current_dir = g_get_current_dir ();
+			comm->filename = g_strconcat (current_dir,
+						      "/",
+						      filename,
+						      NULL);
+			g_free (current_dir);
+		}
+		else
+			comm->filename = g_strdup (filename);
 
-	comm->e_filename = shell_escape (comm->filename);
+		comm->e_filename = g_shell_quote (comm->filename);
 
-	debug (DEBUG_INFO, "filename : %s\n", comm->filename);
-	debug (DEBUG_INFO, "e_filename : %s\n", comm->e_filename);
+		debug (DEBUG_INFO, "filename : %s\n", comm->filename);
+		debug (DEBUG_INFO, "e_filename : %s\n", comm->e_filename);
+	}
+
+	fr_command_working_archive (comm, comm->filename);
 }
 
 
 void
-fr_command_list (FrCommand  *comm,
-		 const char *password)
+fr_command_set_multi_volume (FrCommand  *comm,
+			     const char *filename)
+{
+	comm->multi_volume = TRUE;
+	fr_command_set_filename (comm, filename);
+}
+
+
+void
+fr_command_list (FrCommand  *comm)
 {
 	g_return_if_fail (FR_IS_COMMAND (comm));
 
@@ -496,85 +590,87 @@ fr_command_list (FrCommand  *comm,
 	comm->action = FR_ACTION_LISTING_CONTENT;
 	fr_process_set_out_line_func (FR_COMMAND (comm)->process, NULL, NULL);
 	fr_process_set_err_line_func (FR_COMMAND (comm)->process, NULL, NULL);
-	
+	fr_process_use_standard_locale (FR_COMMAND (comm)->process, TRUE);
+	comm->multi_volume = FALSE;
+
 	if (!comm->fake_load)
-		FR_COMMAND_GET_CLASS (G_OBJECT (comm))->list (comm, password);
+		FR_COMMAND_GET_CLASS (G_OBJECT (comm))->list (comm);
 }
 
 
 void
 fr_command_add (FrCommand     *comm,
+		const char    *from_file,
 		GList         *file_list,
 		const char    *base_dir,
 		gboolean       update,
-		const char    *password,
-		FrCompression  compression)
+		gboolean       recursive)
 {
 	fr_command_progress (comm, -1.0);
 
 	comm->action = FR_ACTION_ADDING_FILES;
 	fr_process_set_out_line_func (FR_COMMAND (comm)->process, NULL, NULL);
 	fr_process_set_err_line_func (FR_COMMAND (comm)->process, NULL, NULL);
-	
+
 	FR_COMMAND_GET_CLASS (G_OBJECT (comm))->add (comm,
+						     from_file,
 						     file_list,
 						     base_dir,
 						     update,
-						     password,
-						     compression);
+						     recursive);
 }
 
 
 void
-fr_command_delete (FrCommand *comm,
-		   GList *file_list)
+fr_command_delete (FrCommand   *comm,
+		   const char  *from_file,
+		   GList       *file_list)
 {
 	fr_command_progress (comm, -1.0);
 
 	comm->action = FR_ACTION_DELETING_FILES;
 	fr_process_set_out_line_func (FR_COMMAND (comm)->process, NULL, NULL);
 	fr_process_set_err_line_func (FR_COMMAND (comm)->process, NULL, NULL);
-	
-	FR_COMMAND_GET_CLASS (G_OBJECT (comm))->delete (comm, file_list);
+
+	FR_COMMAND_GET_CLASS (G_OBJECT (comm))->delete (comm, from_file, file_list);
 }
 
 
 void
 fr_command_extract (FrCommand  *comm,
+		    const char *from_file,
 		    GList      *file_list,
 		    const char *dest_dir,
 		    gboolean    overwrite,
 		    gboolean    skip_older,
-		    gboolean    junk_paths,
-		    const char *password)
+		    gboolean    junk_paths)
 {
 	fr_command_progress (comm, -1.0);
 
 	comm->action = FR_ACTION_EXTRACTING_FILES;
 	fr_process_set_out_line_func (FR_COMMAND (comm)->process, NULL, NULL);
 	fr_process_set_err_line_func (FR_COMMAND (comm)->process, NULL, NULL);
-	
+
 	FR_COMMAND_GET_CLASS (G_OBJECT (comm))->extract (comm,
-							 file_list, 
+							 from_file,
+							 file_list,
 							 dest_dir,
 							 overwrite,
 							 skip_older,
-							 junk_paths,
-							 password);
+							 junk_paths);
 }
 
 
 void
-fr_command_test (FrCommand   *comm,
-		 const char  *password)
+fr_command_test (FrCommand *comm)
 {
 	fr_command_progress (comm, -1.0);
 
 	comm->action = FR_ACTION_TESTING_ARCHIVE;
 	fr_process_set_out_line_func (FR_COMMAND (comm)->process, NULL, NULL);
 	fr_process_set_err_line_func (FR_COMMAND (comm)->process, NULL, NULL);
-	
-	FR_COMMAND_GET_CLASS (G_OBJECT (comm))->test (comm, password);
+
+	FR_COMMAND_GET_CLASS (G_OBJECT (comm))->test (comm);
 }
 
 
@@ -587,24 +683,37 @@ fr_command_uncompress (FrCommand *comm)
 
 
 void
-fr_command_recompress (FrCommand     *comm,
-		       FrCompression  compression)
+fr_command_recompress (FrCommand *comm)
 {
-	fr_command_progress (comm, -1.0);	
-	FR_COMMAND_GET_CLASS (G_OBJECT (comm))->recompress (comm, compression);
+	fr_command_progress (comm, -1.0);
+	FR_COMMAND_GET_CLASS (G_OBJECT (comm))->recompress (comm);
 }
 
 
-char *
-fr_command_escape (FrCommand  *comm,
-		   const char *str)
+const char **
+fr_command_get_mime_types (FrCommand *comm)
 {
-	return FR_COMMAND_GET_CLASS (G_OBJECT (comm))->escape (comm, str);
+	return FR_COMMAND_GET_CLASS (G_OBJECT (comm))->get_mime_types (comm);
+}
+
+
+void
+fr_command_update_capabilities (FrCommand *comm)
+{
+	comm->capabilities = fr_command_get_capabilities (comm, comm->mime_type);
+}
+
+
+FrCommandCap
+fr_command_get_capabilities (FrCommand  *comm,
+			     const char *mime_type)
+{
+	return FR_COMMAND_GET_CLASS (G_OBJECT (comm))->get_capabilities (comm, mime_type);
 }
 
 
 gboolean
-fr_command_is_capable_of (FrCommand     *comm, 
+fr_command_is_capable_of (FrCommand     *comm,
 			  FrCommandCaps  requested_capabilities)
 {
 	return (((comm->capabilities ^ requested_capabilities) & requested_capabilities) == 0);
@@ -612,17 +721,17 @@ fr_command_is_capable_of (FrCommand     *comm,
 
 
 /* fraction == -1 means : I don't known how much time the current operation
- *                        will take, the dialog will display this info pulsing 
- *                        the progress bar. 
- * fraction in [0.0, 1.0] means the amount of work, in percentage, 
- *                        accomplished. 
+ *                        will take, the dialog will display this info pulsing
+ *                        the progress bar.
+ * fraction in [0.0, 1.0] means the amount of work, in percentage,
+ *                        accomplished.
  */
 void
 fr_command_progress (FrCommand *comm,
 		     double     fraction)
 {
 	g_signal_emit (G_OBJECT (comm),
-		       fr_command_signals[PROGRESS], 
+		       fr_command_signals[PROGRESS],
 		       0,
 		       fraction);
 }
@@ -633,12 +742,23 @@ fr_command_message (FrCommand  *comm,
 		    const char *msg)
 {
 	g_signal_emit (G_OBJECT (comm),
-		       fr_command_signals[MESSAGE], 
+		       fr_command_signals[MESSAGE],
 		       0,
 		       msg);
 }
 
 
+void
+fr_command_working_archive (FrCommand  *comm,
+		            const char *archive_name)
+{
+	g_signal_emit (G_OBJECT (comm),
+		       fr_command_signals[WORKING_ARCHIVE],
+		       0,
+		       archive_name);
+}
+
+
 void
 fr_command_set_n_files (FrCommand *comm,
 			int        n_files)
@@ -654,6 +774,8 @@ fr_command_add_file (FrCommand *comm,
 {
 	file_data_update_content_type (fdata);
 	g_ptr_array_add (comm->files, fdata);
+	if (! fdata->dir)
+		comm->n_regular_files++;
 }
 
 
@@ -666,7 +788,7 @@ fr_command_set_mime_type (FrCommand  *comm,
 
 
 void
-fr_command_handle_error (FrCommand *comm,
+fr_command_handle_error (FrCommand   *comm,
 			 FrProcError *error)
 {
 	FR_COMMAND_GET_CLASS (G_OBJECT (comm))->handle_error (comm, error);
